import requests


class PyDevTo:
    def __init__(self, api_key=None, timeout=None):
        """

        :param api_key:  Your dev.to api key (https://dev.to/settings/account)
        :param timeout: Timeout period for http requests
        """
        self.api_key = api_key
        self.timeout = timeout

    def public_articles(self, page=None, tag=None, username=None, state=None, top=None):
        """Return a list of public (published) articles

        :param page: pagination page
        :param tag: articles that contain the requested tag.
        :param username: articles belonging to a User or Organization ordered by descending published_at
        :param state: "fresh" or "rising".  check which articles are fresh or rising.
        :param top: (int) most popular articles in the last N days
        :return:
        """
        return requests.get(
            "https://dev.to/api/articles",
            params={
                "page": page,
                "tag": tag,
                "username": username,
                "state": state,
                "top": top,
            },
        ).json()

    def public_article(self, id):
        """Return a single public (published) article given its id

        :param id: id of the article
        :return: article
        """
        return requests.get(
            "https://dev.to/api/articles/{id}".format(id=id), timeout=self.timeout
        ).json()

    def articles(self, page=None, per_page=None, state="published"):
        """Return a list of user articles

        :param page: pagination page
        :param per_page: page size
        :param state: "published", "unpublished" or "all
        :return: list of articles
        """
        url = "https://dev.to/api/articles/me"
        if state == "published":
            url = "https://dev.to/api/articles/me"
        elif state == "unpublished":
            url = "https://dev.to/api/articles/me/unpublished"
        elif state == "all":
            url = "https://dev.to/api/articles/me/all"

        return requests.get(
            url,
            params={"page": page, "per_page": per_page},
            headers={"api-key": self.api_key},
            timeout=self.timeout,
        ).json()

    def create_article(
        self,
        title,
        body_markdown="",  # must default to empty string instead of None otherwise dev.to raises error on edit
        published=None,
        series=None,
        main_image=None,
        canonical_url=None,
        description=None,
        tags=None,
        organization_id=None,
    ):
        """Create an article

        :param title: Title
        :param body_markdown: Article Markdown content
        :param published: True to create published article, false otherwise
        :param series: Article series name
        :param main_image: Main image (or cover image)
        :param canonical_url: Canonical Url
        :param description: Article Description
        :param tags: List of article tags
        :param organization_id: Organization id
        :return: newly created article
        """
        url = "https://dev.to/api/articles"

        data = {
            "title": title,
            "body_markdown": body_markdown,
            "series": series,
            "published": published,
            "main_image": main_image,
            "canonical_url": canonical_url,
            "description": description,
            "tags": tags,
            "organization_id": organization_id,
        }
        # remove None keys from dict
        data = {k: v for k, v in data.items() if v is not None}

        return requests.post(
            url, json=data, headers={"api-key": self.api_key}, timeout=self.timeout
        ).json()

    def update_article(
        self,
        id,
        title=None,
        body_markdown=None,
        published=None,
        series=None,
        main_image=None,
        canonical_url=None,
        description=None,
        tags=None,
        organization_id=None,
    ):
        """Update an article

        :param id: id of article to update
        :param title: Title
        :param body_markdown: Article Markdown content
        :param published: True to create published article, false otherwise
        :param series: Article series name
        :param main_image: Main image (or cover image)
        :param canonical_url: Canonical Url
        :param description: Article Description
        :param tags: List of article tags
        :param organization_id: Organization id
        :return: updated article
        """
        url = "https://dev.to/api/articles/{id}".format(id=id)

        data = {
            "title": title,
            "body_markdown": body_markdown,
            "series": series,
            "published": published,
            "main_image": main_image,
            "canonical_url": canonical_url,
            "description": description,
            "tags": tags,
            "organization_id": organization_id,
        }
        # remove None keys from dict
        data = {k: v for k, v in data.items() if v is not None}

        return requests.put(
            url, json=data, headers={"api-key": self.api_key}, timeout=self.timeout
        ).json()

    def user(self, id=None, username=None):
        """Return user information

        If both id and username is None then information for user with the api_key (me) is returned

        :param id: (optional) id of user
        :param username: (optional) username of user
        :return: user object
        """
        url = "https://dev.to/api/users/me"
        if id:
            url = "https://dev.to/api/users/{id}".format(id=id)
        elif username:
            url = "https://dev.to/api/users/by_username"

        return requests.get(
            url,
            params={"url": username},
            headers={"api-key": self.api_key},
            timeout=self.timeout,
        ).json()

    def follow_suggestions(self, page=None):
        """Return list of follow suggestions

        :param page: pagination page
        :return: list of follow suggestions
        """
        return requests.get(
            "https://dev.to/api/users/?state=follow_suggestions",
            params={"page": page},
            headers={"api-key": self.api_key},
            timeout=self.timeout,
        ).json()

    def tags(self, page=None):
        """Return list of tags

        :param page: pagination page
        :return:
        """
        return requests.get(
            "https://dev.to/api/tags",
            params={"page": page},
            headers={"api-key": self.api_key},
            timeout=self.timeout,
        ).json()

    def webhooks(self):
        """Return list of webhooks

        :return: list of webhooks
        """
        return requests.get(
            "https://dev.to/api/webhooks",
            headers={"api-key": self.api_key},
            timeout=self.timeout,
        ).json()

    def webhook(self, id):
        """Return single webhook with id

        :param id: id of webhook
        :return: webhook object
        """
        return requests.get(
            "https://dev.to/api/webhooks/{id}".format(id=id),
            headers={"api-key": self.api_key},
            timeout=self.timeout,
        ).json()

    def create_webhook(self, source, target_url, events):
        """Create a new webhook

        :param source: The name of the requester, eg. "DEV"
        :param target_url: Target Url
        :param events: List of event identifiers
        :return:
        """
        return requests.post(
            "https://dev.to/api/webhooks",
            headers={"api-key": self.api_key},
            json={"source": source, "target_url": target_url, "events": events},
            timeout=self.timeout,
        ).json()

    def delete_webhook(self, id):
        """Delete  a webhook with id

        :param id: id of webhook
        :return:
        """
        return requests.delete(
            "https://dev.to/api/webhooks/{id}".format(id=id),
            headers={"api-key": self.api_key},
            timeout=self.timeout,
        ).json()



import urllib.parse as urlparse

from bs4 import BeautifulSoup, NavigableString
import re
import six


convert_heading_re = re.compile(r"convert_h(\d+)")
line_beginning_re = re.compile(r"^", re.MULTILINE)
whitespace_re = re.compile(r"[\r\n\s\t ]+")
FRAGMENT_ID = "__MARKDOWNIFY_WRAPPER__"
wrapped = '<div id="%s">%%s</div>' % FRAGMENT_ID


# Heading styles
ATX = "atx"
ATX_CLOSED = "atx_closed"
UNDERLINED = "underlined"
SETEXT = UNDERLINED


def escape(text):
    if not text:
        return ""
    return text.replace("_", r"\_")


def _todict(obj):
    return dict((k, getattr(obj, k)) for k in dir(obj) if not k.startswith("_"))


class MarkdownConverter(object):
    """
    This class is based on on markdownify (https://github.com/matthewwithanm/python-markdownify) but extended to
    handle dev.to specific markdown and embeds
    """
    class DefaultOptions:
        strip = None
        convert = None
        autolinks = True
        heading_style = UNDERLINED
        bullets = "*+-"  # An iterable of bullet types.

    class Options(DefaultOptions):
        pass

    def __init__(self, **options):
        # Create an options dictionary. Use DefaultOptions as a base so that
        # it doesn't have to be extended.
        self.options = _todict(self.DefaultOptions)
        self.options.update(_todict(self.Options))
        self.options.update(options)
        if self.options["strip"] is not None and self.options["convert"] is not None:
            raise ValueError(
                "You may specify either tags to strip or tags to"
                " convert, but not both."
            )

    def convert(self, html):
        # We want to take advantage of the html5 parsing, but we don't actually
        # want a full document. Therefore, we'll mark our fragment with an id,
        # create the document, and extract the element with the id.
        html = wrapped % html
        soup = BeautifulSoup(html, "html.parser")
        return self.process_tag(soup.find(id=FRAGMENT_ID), children_only=True)

    def process_tag(self, node, children_only=False):
        text = ""

        # Convert the children first
        for el in node.children:
            if isinstance(el, NavigableString):
                text += self.process_text(six.text_type(el))
            else:
                text += self.process_tag(el)

        if not children_only:
            convert_fn = getattr(self, "convert_%s" % node.name, None)
            if convert_fn and self.should_convert_tag(node.name):
                text = convert_fn(node, text)
            # else:
            #     text = text + node.name

        return text

    def process_text(self, text):
        return escape(whitespace_re.sub(" ", text or ""))

    def __getattr__(self, attr):
        # Handle headings
        m = convert_heading_re.match(attr)
        if m:
            n = int(m.group(1))

            def convert_tag(el, text):
                return self.convert_hn(n, el, text)

            convert_tag.__name__ = "convert_h%s" % n
            setattr(self, convert_tag.__name__, convert_tag)
            return convert_tag

        raise AttributeError(attr)

    def should_convert_tag(self, tag):
        tag = tag.lower()
        strip = self.options["strip"]
        convert = self.options["convert"]
        if strip is not None:
            return tag not in strip
        elif convert is not None:
            return tag in convert
        else:
            return True

    def indent(self, text, level):
        return line_beginning_re.sub("\t" * level, text) if text else ""

    def underline(self, text, pad_char):
        text = (text or "").rstrip()
        return "%s\n%s\n\n" % (text, pad_char * len(text)) if text else ""

    def convert_a(self, el, text):
        href = el.get("href")
        title = el.get("title")
        if self.options["autolinks"] and text == href and not title:
            # Shortcut syntax
            return "<%s>" % href
        title_part = ' "%s"' % title.replace('"', r"\"") if title else ""
        return "[%s](%s%s)" % (text or "", href, title_part) if href else text or ""

    def convert_b(self, el, text):
        return self.convert_strong(el, text)

    def convert_figcaption(self, el, text):
        return "\n<figcaption>" + text + "</figcaption>\n"

    def convert_blockquote(self, el, text):

        # handle twitter embeds
        if el.get("class") and el.get("class")[0] == "twitter-tweet":
            for entry in el.find_all("a"):
                if entry.get("href").startswith("https://twitter.com/"):
                    return self.pydevto_embed(entry.get("href"))

        return "\n" + line_beginning_re.sub("> ", text) if text else ""

    def convert_br(self, el, text):
        return "  \n"

    def remove_scheme(self, url):
        return (
            url.replace("https://www.", "//")
            .replace("https://", "//")
            .replace("http://www.", "//")
            .replace("http://", "//")
        )

    def convert_iframe(self, el, text):
        src = self.remove_scheme(el.attrs.get("src"))
        if src.startswith("//cdn.unfurl.dev/embed?"):
            src = urlparse.unquote(src)
            query = urlparse.urlsplit(src).query
            params = urlparse.parse_qs(query)
            if params.get("url"):
                embed = params.get("url")[0]
                return self.pydevto_embed(embed)

        if src.startswith("//cdn.embedly.com"):
            src = urlparse.unquote(src)
            query = urlparse.urlsplit(src).query
            params = urlparse.parse_qs(query)
            if params.get("src"):
                embed = params.get("src")[0]
                return self.pydevto_embed(embed)

        return self.pydevto_embed(el.attrs.get("src"))

    def pydevto_embed(self, url):
        original_url = url
        url = self.remove_scheme(url)

        if url.startswith("//twitter.com"):
            q = urlparse.urlsplit(url)
            return "\n{% twitter " + q.path.split("/")[-1] + " %}\n"

        if url.startswith("//youtube.com"):
            q = urlparse.urlsplit(url)
            if q.path.startswith("/embed/"):
                return "\n{% youtube " + q.path.replace("/embed/", "") + " %}\n"
            v = urlparse.parse_qs(q.query)["v"][0]
            return "\n{% youtube " + v + " %}\n"

        if url.startswith("//codepen.io"):
            return "\n{% codepen " + original_url + " %}\n"

        if url.startswith("//soundcloud.com"):
            return "\n{% soundcloud " + original_url + " %}\n"

        if url.startswith("//github.com"):
            return "\n{% github " + original_url + " %}\n"

        if url.startswith("//instagram.com"):
            q = urlparse.urlsplit(url)
            return (
                "\n{% instagram " + q.path.replace("/p", "").replace("/", "") + " %}\n"
            )

        if url.startswith("//repl.it"):
            q = urlparse.urlsplit(url)
            return "\n{% replit " + q.path[1:] + " %}\n"

        return "\n" + original_url + "\n"

    def convert_em(self, el, text):
        return "*%s*" % text if text else ""

    def convert_hn(self, n, el, text):
        style = self.options["heading_style"]
        text = text.rstrip()
        if style == UNDERLINED and n <= 2:
            line = "=" if n == 1 else "-"
            return self.underline(text, line)
        hashes = "#" * n
        if style == ATX_CLOSED:
            return "%s %s %s\n\n" % (hashes, text, hashes)
        return "%s %s\n\n" % (hashes, text)

    def convert_i(self, el, text):
        return self.convert_em(el, text)

    def convert_list(self, el, text):
        nested = False
        while el:
            if el.name == "li":
                nested = True
                break
            el = el.parent
        if nested:
            text = "\n" + self.indent(text, 1)
        return "\n" + text + "\n"

    convert_ul = convert_list
    convert_ol = convert_list

    def convert_li(self, el, text):
        parent = el.parent
        if parent is not None and parent.name == "ol":
            bullet = "%s." % (parent.index(el) + 1)
        else:
            depth = -1
            while el:
                if el.name == "ul":
                    depth += 1
                el = el.parent
            bullets = self.options["bullets"]
            bullet = bullets[depth % len(bullets)]
        return "%s %s\n" % (bullet, text or "")

    def convert_p(self, el, text):
        return "%s\n\n" % text if text else ""

    def convert_strong(self, el, text):
        return "**%s**" % text if text else ""

    def convert_img(self, el, text):
        alt = el.attrs.get("alt", None) or ""
        src = el.attrs.get("src", None) or ""
        title = el.attrs.get("title", None) or ""
        title_part = ' "%s"' % title.replace('"', r"\"") if title else ""
        return "![%s](%s%s)" % (alt, src, title_part)


def html_to_markdown(html):
    html = html.replace("<hr>", "<p>---</p>")
    return MarkdownConverter(heading_style="atx").convert(html)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['pydevto']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.8,<5.0', 'requests>=2.22,<3.0', 'six>=1.12,<2.0']

setup_kwargs = {
    'name': 'pydevto',
    'version': '0.1.4',
    'description': 'Unofficial dev.to api for python',
    'long_description': '# PyDevTo\n\nUnofficial dev.to api for python.\n\n### Features\n* Implements all endpoints from https://docs.dev.to/api/\n* Implements a few other api endpoints not documented but available in the source, such as users and follow_suggestions\n* Includes a helper method to convert html to dev.to specific markdown, including support for dev.to specific embeds such as YouTube.\n\n\n## Installation\n\nUse the package manager [pip](https://pip.pypa.io/en/stable/) to install pydevto.\n\n```bash\npip install pydevto\n```\n\n## Usage\n\nMake sure you have an api key to use the authenticated endpoints.  You can get your key from https://dev.to/settings/account\n(You can use pydevto without an api key for some functions, such as the public articles)\n\n```python\nimport pydevto\napi = pydevto.PyDevTo(api_key=\'MY_KEY\')\napi.articles()  # returns list of your own published articles\n```\n\n## Methods\n```python\nimport pydevto\napi = pydevto.PyDevTo(api_key=\'MY_KEY\')\napi.public_articles(page=None, tag=None, username=None, state=None, top=None)  # Return list of public (published) articles\napi.public_article(id)  # Return a single public (published) article given its id\napi.articles(page=None, per_page=None, state="published")  # Return a list of user articles\napi.create_article(...)  # Create an article\napi.update_article(id, ...)  # Update an article\napi.user(id=None, username=None)  # Return user information\napi.follow_suggestions(page=None)  # Return list of follow suggestions\napi.tags(page=None)  # Return list of tags\napi.webhooks()  # Return list of webhooks\napi.webhook(id)  # Return single webhook with id\napi.create_webhook(source, target_url, events)  # Create a new webhook\napi.delete_webhook(id)  # Delete  a webhook with id\n```\n\n## Html to Markdown\nPyDevTo contains a helper function to convert html to dev.to specific markdown (https://dev.to/p/editor_guide)\nIt supports images with captions using the HTML figcaption tag, and converts embeds such as YouTube to dev.to specific liquid tags.\n```python\n>>> import pydevto\n>>> pydevto.html_to_markdown(\'<h1>Heading</h1\') \n>>> \'# Heading\\n\\n\'\n>>> pydevto.html_to_markdown(\'<iframe src="https://www.youtube.com/embed/kmjiUVEMvI4"></iframe>\') \n>>> \'\\n{% youtube kmjiUVEMvI4 %}\\n\'  \n```\n\n## Known issues\n* The tags property does not currently work correctly when creating/updating an article.  There is an open issue report on dev.to for this.\n* The html to markdown only caters for a subset of embeds (YouTube, Twitter, repl.it, soundcloud and a few more), more will be added over time.\n\n## Contributing\nPull requests and issue reports are welcome.\n\n## License\n[MIT](https://choosealicense.com/licenses/mit/)\n',
    'author': "'Loftie",
    'author_email': 'lpellis@gmail.com',
    'url': 'https://github.com/lpellis/pydevto',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
